@@ -2,9 +2,10 @@ from pathlib import Path
 import yaml
 import typer
 from logger_tt import logger
+import psutil
 
 
-def load_config(config_path=None):
+def load_config(config_path=None, log_port=None, log_path=None):
     if config_path is None:
         config_path = "config.yaml"
 
@@ -20,9 +21,41 @@ def load_config(config_path=None):
         logger.error("--forward_only is not implemented yet.")
         raise typer.Abort()
 
+    if log_port is not None:
+        config["log_port"] = log_port
+
+    if log_path is not None:
+        config["log_path"] = log_path
+
+    if "cores_pr_fit" not in config:
+        config["cores_pr_fit"] = 1
+
     return config
 
 
+def check_number_of_jobs(config):
+
+    cores = config["cores"]
+    cores_pr_fit = config["cores_pr_fit"]
+    N_jobs = cores * cores_pr_fit
+    max_cores = psutil.cpu_count(logical=True)
+    max_cores_real = psutil.cpu_count(logical=False)
+
+    if N_jobs > max_cores:
+        logger.warning(
+            f"The total number of jobs {N_jobs} are higher "
+            f"than the number of cores {max_cores}. "
+            "Do not do this unless you know what you are doing. "
+            "Try decreasing either 'cores' or 'cores-pr-fit'."
+        )
+    elif N_jobs > max_cores_real:
+        logger.info(
+            f"The total number of jobs {N_jobs} are higher "
+            f"than the real number of cores {max_cores} (non-logical). "
+            "This might decrease performance. "
+        )
+
+
 def path_endswith(path, s):
     return str(path.name).endswith(s)
 
@@ -111,13 +144,14 @@ def get_results_dir(config_path=None, results_dir=None):
 
     if config_path is not None and results_dir is not None:
         raise AssertionError(
-            "Only a single one of 'config_path' and 'results_dir' can be set"
+            "Only a single one of 'config' and 'results_dir' can be set"
         )
 
-    if results_dir is not None:
-        pass
+    results_dir = Path(results_dir) if isinstance(results_dir, str) else results_dir
+    config_path = Path(config_path) if isinstance(config_path, str) else config_path
 
-    else:
-        results_dir = Path(load_config(config_path)["dir"]) / "results"
+    if isinstance(results_dir, Path):
+        return results_dir
 
+    results_dir = Path(load_config(config_path)["dir"]) / "results"
     return results_dir
